from fastapi import FastAPI, Body, Depends
from .auth import sign_up, sign_in, create_api_key, authenticate

users_app = FastAPI(
    title="Fyodorov-Auth",
    description="Common auth endpoints for Fyodorov services",
    version="0.0.1",
)


@users_app.get("/")
async def root(user=Depends(authenticate)):
    print(f"[/users/] User: {user}")
    return {"message": "logged in"}


@users_app.post("/sign_up")
async def sign_up_endpoint(
    email: str = Body(...), password: str = Body(...), invite_code: str = Body(...)
):
    return await sign_up(email, password, invite_code)


@users_app.post("/sign_in")
async def sign_in_endpoint(email: str = Body(...), password: str = Body(...)):
    return await sign_in(email, password)


@users_app.post("/create_api_key")
async def create_api_key_endpoint(expiration: int = 15, user=Depends(authenticate)):
    return await create_api_key(expiration, user)



from pydantic_settings import BaseSettings
import os
from dotenv import load_dotenv

load_dotenv()


class Settings(BaseSettings):
    JWT_SECRET: str = os.getenv("JWT_SECRET")
    SUPABASE_URL: str = os.getenv("SUPABASE_PROJECT_URL")
    SUPABASE_KEY: str = os.getenv("SUPABASE_API_KEY")



import os

# Service name to port mapping
SERVICE_PORTS = {
    'Dostoyevsky': 8000,
    'Zhukovsky': 8003, 
    'Gagarin': 8002,
    'Tsiolkovsky': 8001
}

def get_service_url(service_name: str) -> str:
    """Get service URL from environment or default to Docker Compose service name."""
    if service_name not in SERVICE_PORTS:
        raise ValueError(f"Unknown service: {service_name}")
    
    env_var = f"{service_name.upper()}_URL"
    default_url = f"http://{service_name.lower()}:{SERVICE_PORTS[service_name]}"
    
    return os.getenv(env_var, default_url)



from fastapi import FastAPI, Depends, Request, HTTPException
from fastapi.responses import Response
import yaml

from fyodorov_utils.decorators.logging import error_handler
from fyodorov_utils.auth.auth import authenticate

from fyodorov_llm_agents.providers.provider_model import ProviderModel
from fyodorov_llm_agents.providers.provider_service import Provider
from fyodorov_llm_agents.models.llm_model import LLMModel
from fyodorov_llm_agents.tools.mcp_tool_service import MCPTool as Tool
from fyodorov_llm_agents.tools.mcp_tool_model import MCPTool as ToolModel
from fyodorov_llm_agents.instances.instance_model import InstanceModel
from fyodorov_llm_agents.models.llm_service import LLMService
from fyodorov_llm_agents.instances.instance_service import Instance
from fyodorov_llm_agents.agents.agent_service import AgentService as Agent

app = FastAPI(
    title="Fyodorov-Auth",
    description="Common auth endpoints for Fyodorov services",
    version="0.0.1",
)


# Yaml parsing
@app.post("/")
@error_handler
async def create_from_yaml(request: Request, user=Depends(authenticate)):
    try:
        fyodorov_yaml = await request.body()
        print(f"fyodorov_yaml: \n{fyodorov_yaml}")
        fyodorov_config = yaml.safe_load(fyodorov_yaml)
        print(f"fyodorov_config: \n{fyodorov_config}")
        response = {
            "providers": [],
            "models": [],
            "agents": [],
            "instances": [],
            "tools": [],
        }
        print(f"fyodorov_config: \n{fyodorov_config}")
        if "providers" in fyodorov_config:
            print("Saving providers")
            for provider_dict in fyodorov_config["providers"]:
                print(f"Provider: {provider_dict}")
                provider = ProviderModel.from_dict(provider_dict)
                new_provider = await Provider.save_provider_in_db(
                    user["session_id"], provider, user["sub"]
                )
                response["providers"].append(new_provider)
        print("Saved providers", response["providers"])
        if "models" in fyodorov_config:
            for model_dict in fyodorov_config["models"]:
                model = LLMModel.from_dict(model_dict)
                print(f"Model: {model}")
                llm_service = LLMService()
                new_model = await llm_service.save_model_in_db(
                    user["session_id"], user["sub"], model
                )
                response["models"].append(new_model.to_dict())
        print("Saved models", response["models"])
        if "tools" in fyodorov_config:
            for tool_dict in fyodorov_config["tools"]:
                print(f"Tool dict: {tool_dict}")
                # marshal back to yaml
                tool_yaml = yaml.dump(tool_dict)
                print(f"Tool yaml: {tool_yaml}")
                new_tool = ToolModel.from_yaml(tool_yaml)
                print(f"New tool: {new_tool}")
                if new_tool:
                    tool_instance = await Tool.create_or_update_in_db(
                        user["session_id"], new_tool, user["sub"]
                    )
                    print(f"Saved tool: {tool_instance}")
                    response["tools"].append(tool_instance.to_dict())
        print("Saved tools", response["tools"])
        if "agents" in fyodorov_config:
            for agent_dict in fyodorov_config["agents"]:
                new_agent = await Agent.save_from_dict(
                    user["session_id"], user["sub"], agent_dict
                )
                response["agents"].append(new_agent)
        print("Saved agents", response["agents"])
        if len(response["agents"]) > 0:
            for agent in response["agents"]:
                instance = InstanceModel(
                    agent_id=str(agent["id"]), title=f"Default Instance {agent['id']}"
                )
                new_instance = await Instance.create_in_db(instance)
                response["instances"].append(new_instance)
        print("Saved instances", response["instances"])
        return response
    except Exception as e:
        print("Error parsing config from yaml", str(e))
        raise HTTPException(status_code=400, detail="Invalid YAML format")


@app.get("/")
@error_handler
async def get_yaml(user=Depends(authenticate)):
    try:
        limit = 100
        result = {
            "providers": [],
            "models": [],
            "agents": [],
            "instances": [],
            "tools": [],
        }
        providers = await Provider.get_providers(limit=limit, user_id=user["sub"])
        result["providers"] = [provider.resource_dict() for provider in providers]
        llm_service = LLMService()
        models = await llm_service.get_models(access_token=user["session_id"], user_id=user["sub"], limit=limit)
        result["models"] = [model.resource_dict() for model in models]
        agents = await Agent.get_all_in_db(limit=limit, user_id=user["sub"])
        result["agents"] = [agent.resource_dict() for agent in agents]
        instances = await Instance.get_all_in_db(limit=limit, user_id=user["sub"])
        result["instances"] = [instance.resource_dict() for instance in instances]
        tools = await Tool.get_all_in_db(limit=limit, user_id=user["sub"])
        result["tools"] = [tool.resource_dict() for tool in tools]
        print(f"Result: {result}")
        yaml_result = yaml.dump(result, indent=2)
        print(f"YAML: {yaml_result}")
        return Response(content=yaml_result, media_type="application/x-yaml")
    except Exception as e:
        print("Error getting yaml:", str(e))
        raise HTTPException(
            status_code=400, detail="Error marshaling resources to yaml"
        )


@app.get("/{resource_type}")
@error_handler
async def get_yaml_by_name(resource_type: str, user=Depends(authenticate)):
    limit = 100
    print(f"Got request for {resource_type} yaml")
    resources = {}
    try:
        if resource_type not in ["providers", "models", "agents", "instances", "tools"]:
            raise HTTPException(status_code=400, detail="Unrecognized resource type")
        elif resource_type == "providers":
            resources["providers"] = await Provider.get_providers(
                limit=limit, user_id=user["sub"]
            )
            resources["providers"] = [
                provider.resource_dict() for provider in resources["providers"]
            ]
        elif resource_type == "models":
            llm_service = LLMService()
            resources["models"] = await llm_service.get_models(access_token=user["session_id"], user_id=user["sub"], limit=limit)
            resources["models"] = [
                models.resource_dict() for models in resources["models"]
            ]
        elif resource_type == "agents":
            resources["agents"] = await Agent.get_all_in_db(
                limit=limit, user_id=user["sub"]
            )
            resources["agents"] = [
                agents.resource_dict() for agents in resources["agents"]
            ]
        elif resource_type == "instances":
            resources["instances"] = await Instance.get_all_in_db(
                limit=limit, user_id=user["sub"]
            )
            resources["instances"] = [
                instances.resource_dict() for instances in resources["instances"]
            ]
        elif resource_type == "tools":
            resources["tools"] = await Tool.get_all_in_db(
                limit=limit, user_id=user["sub"]
            )
            resources["tools"] = [tools.resource_dict() for tools in resources["tools"]]
        else:
            raise HTTPException(status_code=400, detail="Invalid resource type")
        print(f"Resources: {resources}")
        yaml_result = yaml.dump(resources, indent=2)
        return Response(content=yaml_result, media_type="application/x-yaml")
    except Exception as e:
        print("Error getting yaml for resource:", str(e))
        raise HTTPException(
            status_code=400,
            detail=f"Error marshaling {resource_type} resources to yaml",
        )



from datetime import datetime
from fyodorov_utils.config.supabase import get_supabase
from fyodorov_llm_agents.tools.tool import Tool as ToolModel
from .base import Base


class Tool(Base):
    def create_from_plugin(access_token: str, plugin: dict) -> ToolModel:
        print("in func creating tool from plugin")
        tool = ToolModel.from_plugin(plugin)
        print("created tool from plugin", tool)
        return Tool.create_in_db(access_token, tool)

    @staticmethod
    def create_in_db(access_token: str, tool: ToolModel) -> str:
        try:
            supabase = get_supabase(access_token)
            result = supabase.table("tools").insert(tool.to_dict()).execute()
            tool_id = result.data[0]["id"]
            return tool_id
        except Exception as e:
            print("Error creating tool", str(e))
            raise e

    @staticmethod
    def update_in_db(access_token: str, id: str, tool: dict) -> dict:
        if not id:
            raise ValueError("Tool ID is required")
        try:
            supabase = get_supabase(access_token)
            result = supabase.table("tools").update(tool).eq("id", id).execute()
            return result.data[0]
        except Exception as e:
            print("An error occurred while updating tool:", id, str(e))
            raise

    @staticmethod
    def delete_in_db(access_token: str, id: str) -> bool:
        if not id:
            raise ValueError("Tool ID is required")
        try:
            supabase = get_supabase(access_token)
            supabase.table("tools").delete().eq("id", id).execute()
            return True
        except Exception as e:
            print("Error deleting tool", str(e))
            raise e

    @staticmethod
    def get_in_db(access_token: str, id: str) -> ToolModel:
        if not id:
            raise ValueError("Tool ID is required")
        try:
            supabase = get_supabase(access_token)
            result = supabase.table("tools").select("*").eq("id", id).limit(1).execute()
            tool_dict = result.data[0]
            print("got tool from db", tool_dict)
            tool = ToolModel(**tool_dict)
            return tool
        except Exception as e:
            print("Error fetching tool", str(e))
            raise e

    @staticmethod
    def get_by_name_and_user_id(
        access_token: str, name: str, user_id: str
    ) -> ToolModel:
        if not name:
            raise ValueError("Tool name is required")
        try:
            supabase = get_supabase(access_token)
            result = (
                supabase.table("tools")
                .select("*")
                .eq("name_for_ai", name)
                .eq("user_id", user_id)
                .limit(1)
                .execute()
            )
            if (
                not result.data
            ):  # If no tools found for this user check for public tools with same name
                result = (
                    supabase.table("tools")
                    .select("*")
                    .eq("name_for_ai", name)
                    .eq("public", True)
                    .limit(1)
                    .execute()
                )
            if not result.data:
                print(f"No tool found with name {name} and user_id {user_id}")
                return None
            tool_dict = result.data[0]
            tool_dict["name"] = tool_dict["name_for_ai"]
            tool_dict["description"] = tool_dict["description_for_ai"]
            print("got tool from db", tool_dict)
            tool = ToolModel(**tool_dict)
            return tool
        except Exception as e:
            print("Error fetching tool", str(e))
            raise e

    @staticmethod
    def get_all_in_db(
        access_token: str, limit: int = 10, created_at_lt: datetime = datetime.now()
    ) -> [dict]:
        try:
            supabase = get_supabase(access_token)
            result = (
                supabase.from_("tools")
                .select("*")
                .limit(limit)
                .lt("created_at", created_at_lt)
                .order("created_at", desc=True)
                .execute()
            )
            tools = result.data
            return tools
        except Exception as e:
            print("Error fetching tools", str(e))
            raise e



from datetime import datetime
from fyodorov_utils.config.supabase import get_supabase


class Model:
    id: str = None

    def to_dict(self):
        return {c.name: str(getattr(self, c.name)) for c in self.__table__.columns}

    def from_dict(self, data):
        for field in self.__table__.columns:
            if field.name in data:
                setattr(self, field.name, data[field.name])


class Base(Model):
    table: str = "ERROR: table not set"

    def create_in_db(self, access_token: str) -> str:
        try:
            supabase = get_supabase(access_token)
            result = supabase.table(self.table).insert(self.to_dict()).execute()
            self.id = result.data[0]["id"]
            return self.id
        except Exception as e:
            print("Error creating object in table", self.table, str(e))
            raise e

    def update_in_db(self, access_token: str, id: str) -> dict:
        if not id:
            raise ValueError("ID is required to update in table", self.table)
        try:
            self.id = id
            supabase = get_supabase(access_token)
            result = (
                supabase.table(self.table)
                .update(self.to_dict())
                .eq("id", self.id)
                .execute()
            )
            return result.data[0]
        except Exception as e:
            print("An error occurred while updating table", self.table, id, str(e))
            raise

    def delete_in_db(self, access_token: str, id: str) -> bool:
        if not id:
            raise ValueError("ID is required to delete from table", self.table)
        try:
            self.id = id
            supabase = get_supabase(access_token)
            supabase.table(self.table).delete().eq("id", self.id).execute()
            return True
        except Exception as e:
            print("Error deleting from table", self.table, str(e))
            raise e

    def get_in_db(self, access_token: str, id: str) -> Model:
        if not id:
            raise ValueError("ID is required to get from table", self.table)
        try:
            self.id = id
            supabase = get_supabase(access_token)
            result = (
                supabase.table(self.table)
                .select("*")
                .eq("id", self.id)
                .limit(1)
                .execute()
            )
            self.from_dict(result.data[0])
            return self.base_object
        except Exception as e:
            print("Error fetching tool", str(e))
            raise e

    def get_all_in_db(
        self,
        access_token: str,
        limit: int = 10,
        created_at_lt: datetime = datetime.now(),
    ) -> [dict]:
        try:
            supabase = get_supabase(access_token)
            result = (
                supabase.from_(self.table)
                .select("*")
                .limit(limit)
                .lt("created_at", created_at_lt)
                .order("created_at", desc=True)
                .execute()
            )
            tools = result.data
            return tools
        except Exception as e:
            print("Error fetching multiple objects from table", self.table, str(e))
            raise e
